-- This code is based off the code from LibCandyBar-3.0


--------------------------------------------------------------------------------
-- Internal functions
--------------------------------------------------------------------------------


-- SecondsToTimeDetail() - Used to format the remaining time
local function SecondsToTimeDetail( t )
	if t >= 3600 then -- > 1 hour
		local h = floor(t/3600)
		local m = t - (h*3600)
		return "%d:%02d", h, m
	elseif t >= 60 then -- 1 minute to 1 hour
		local m = floor(t/60)
		local s = t - (m*60)
		return "%d:%02d", m, s
	elseif t < 10 then -- 0 to 10 seconds
		return "%1.1f", t
	else -- 10 seconds to one minute
		return "%.0f", floor(t + .5)
	end
end


-- OnUpdate() - OnUpdate function used to animate the bar
local function OnUpdate(self)
	-- Check if the time expired
	local t = GetTime()
	if t >= self.exp then
		self:Stop()
	else
		-- Update the status bar and duration label
		local time = self.exp - t
		self.remaining = time
		self.statusBar:SetValue(time)		
		self.durationLabel:SetFormattedText(SecondsToTimeDetail(time))
	end
end


--------------------------------------------------------------------------------
-- Local functions to set the bar color, texture, etc..
--------------------------------------------------------------------------------


-- SetBarColor() - Set the bar colors
local SetBarColor = function(self, color, bgcolor)
	self.statusBar:SetStatusBarColor(unpack(color))
	self.background:SetVertexColor(unpack(bgcolor))
end


-- SetBarTexture() - Set the bar textures
local SetBarTexture = function(self, texture)
	self.statusBar:SetStatusBarTexture(texture)
	self.background:SetTexture(texture)
end


-- LockBar() - Lock the bar
local LockBar = function(self)
	self.locked = true
	self:EnableMouse(false)
	self:SetBackdrop(nil)
end


-- UnlockBar() - Unlock the bar
local UnlockBar = function(self)
	self.locked = false
	self:EnableMouse(true)
	self:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"})
	self:SetBackdropColor(1.0, 0.0, 0.0, 0.75)
end


-- StartBar() - Start the bar
local StartBar = function(self, duration)
	self.remaining = duration
	self.running = true
	
	self.statusBar:SetPoint("TOPLEFT", self)
	self.statusBar:SetPoint("BOTTOMLEFT", self)
	self.statusBar:Show()
	self.statusBar:SetMinMaxValues(0, self.remaining)
	
	self.exp = GetTime() + self.remaining
	
	self:SetScript("OnUpdate", OnUpdate)
	self:Show()
end


-- StopBar() - Stop the bar
local StopBar = function(self)
	self.running = nil
	self:SetScript("OnUpdate", nil)
	self.statusBar:Hide()
	
	if not self.locked then
		self:Unlock()
	end
end


--------------------------------------------------------------------------------
-- Global functions
--------------------------------------------------------------------------------


-- CreateBar() - Create the bar
function CreateBar(texture, width, height)

	-- Create the main frame
	local bar = CreateFrame("Frame", "EnragedBar", UIParent)
	bar:ClearAllPoints()
	bar:SetMovable(1)
	bar:SetScale(1)
	bar:SetAlpha(1)
	
	-- Create the statusbar
	bar.statusBar = CreateFrame("StatusBar", nil, bar)
	bar.statusBar:SetPoint("TOPRIGHT")
	bar.statusBar:SetPoint("BOTTOMRIGHT")
	bar.statusBar:SetStatusBarTexture(texture)
	
	-- Create the background texture
	bar.background = bar.statusBar:CreateTexture(nil, "BACKGROUND")
	bar.background:SetVertexColor(0.5, 0.5, 0.5, 0.3)
	bar.background:SetTexture(texture)
	bar.background:SetAllPoints()
	
	local fontName, fontSize = GameFontHighlightSmallOutline:GetFont()
	
	-- Create the duration label
	bar.durationLabel = bar.statusBar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
	bar.durationLabel:SetPoint("RIGHT", bar.statusBar, "RIGHT", -2, 0)
	bar.durationLabel:SetTextColor(1,1,1,1)
	bar.durationLabel:SetJustifyH("CENTER")
	bar.durationLabel:SetJustifyV("MIDDLE")
	bar.durationLabel:SetFont(fontName, fontSize)
	bar.durationLabel:SetFontObject("GameFontHighlightSmallOutline")
	
	-- Create the name label
	bar.nameLabel = bar.statusBar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
	bar.nameLabel:SetPoint("LEFT", bar.statusBar, "LEFT", 2, 0)
	bar.nameLabel:SetPoint("RIGHT", bar.statusBar, "RIGHT", -2, 0)
	bar.nameLabel:SetTextColor(1,1,1,1)
	bar.nameLabel:SetJustifyH("CENTER")
	bar.nameLabel:SetJustifyV("MIDDLE")
	bar.nameLabel:SetFont(fontName, fontSize)
	bar.nameLabel:SetFontObject("GameFontHighlightSmallOutline")
	bar.nameLabel:SetText("Enraged")
	
	-- Set the functions
	bar.Stop = StopBar
	bar.Start = StartBar
	bar.Lock = LockBar
	bar.Unlock = UnlockBar
	bar.SetTexture = SetBarTexture
	bar.SetColor = SetBarColor

	bar.locked = true
	
	return bar
end
